package org.deken.game.sprites;

import org.deken.game.map.GameLocation;

import org.deken.game.animation.Animation;
import org.deken.game.animation.StaticAnimationStrip;
import org.deken.game.collision.CollisionMap;
import org.deken.game.motion.Motion;
import org.deken.game.movement.Movement;

/**
 * <p>Title: Actor</p>
 *
 * <p>Description: A moving sprite.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>@author Gary Deken</p>
 * @version 1.0
 */
public abstract class Actor extends Sprite {
   protected Movement movement;
   protected Motion[] motion;
   protected Animation stillAnimation = null;
   protected int currentMotion = 0;
   protected CollisionMap collisionMap;

   /**
    * Actor
    * This constructor is used by the DocumentManager. It is not recommended to
    * use this outside of that class.
    *
    * @param size PieceSize
    */
   public Actor(SpriteSize size) {
      super(size);
   }

   /**
    * Actor
    *
    * @param motion Motion
    * @param movement Movement
    * @param size SpriteSize
    */
   public Actor(Motion motion, Movement movement, SpriteSize size) {
      this(motion, movement, size, new StaticAnimationStrip(motion.getImage()));
   }

   /**
    * Actor
    *
    * @param motion Motion
    * @param movement Movement
    * @param size SpriteSize
    * @param stillAnimation Animation
    */
   public Actor(Motion motion, Movement movement, SpriteSize size,
      Animation stillAnimation) {
      this(motion, 1, movement, size, stillAnimation);
   }

   public Actor(Motion motion, int numberOfMotions, Movement movement,
      SpriteSize size, Animation stillAnimation) {
      super(size);
      this.motion = new Motion[numberOfMotions];
      this.motion[currentMotion] = motion;
      this.movement = movement;
      this.stillAnimation = stillAnimation;
   }

   public Actor(Motion stillMotion, int numberOfMotions, Movement movement,
      SpriteSize size) {
      super(size);
      this.motion = new Motion[numberOfMotions];
      this.motion[currentMotion] = stillMotion;
      this.movement = movement;
   }

   /**
    * update
    *
    * <p>This method calls the update for the motion, and sets the new
    * location, based on the updated X and Y.</p>
    *
    * @param elaspeTime long
    */
   public void update(long elaspeTime) {
      // first update the movement
      movement.update(elaspeTime);
      // then get the movement's Game Vector and update the motion.
      // update the animation.
      motion[currentMotion].update(elaspeTime, movement.getGameVector());
      double newX = location.getX() + movement.getXUpdate();
      GameLocation gameLocation = collisionMap.getTileCollision(this,
         (float) newX, (float) location.getY());
      if (gameLocation == null) {
         // no collision, update the location
         location.setX(newX);
      } else {
         // Collision. Adjust players location.
         if (movement.getXUpdate() > 0.0) {
            // moving right
            location.setX(collisionMap.getMapSize().getXPixelFromTile((int) gameLocation.getX()) - getSize().getWidth());
         } else if (movement.getXUpdate() < 0.0) {
            // moving left
            location.setX(collisionMap.getMapSize().getXPixelFromTile((int) gameLocation.getX() + 1));
         }
         movement.collideHorizontal();
      }
      Sprite collideSprite = collisionMap.getSpriteCollision(this);
      if (collideSprite != null) {
         spriteCollison(collideSprite);

      }
      double newY = location.getY() + movement.getYUpdate();
      gameLocation = collisionMap.getTileCollision(this,
         (float) location.getX(), (float) newY);
      if (gameLocation == null) {
         location.setY(newY);
      } else {
         if (movement.getYUpdate() > 0.0) {
            // moving down
            location.setY(collisionMap.getMapSize().getYPixelFromTile((int) gameLocation.getY()) - getSize().getHeight());
         } else if (movement.getYUpdate() < 0.0) {
            // moving up
            location.setY(collisionMap.getMapSize().getYPixelFromTile((int) gameLocation.getY() + 1));
         }
         movement.collideVertical();
      }
   }

   /**
    * clone
    *
    * @return Actor
    */
   public Actor cloneBase(Actor newActor) {
      if (this.movement != null) {
         newActor.movement = (Movement) this.movement.clone();
      }
      newActor.motion = new Motion[this.motion.length];
      for (int i = 0; i < newActor.motion.length; i++) {
         newActor.motion[i] = (Motion) this.motion[i].clone();
      }
      if (this.stillAnimation != null) {
         newActor.stillAnimation = (Animation) this.stillAnimation.clone();
      }
      return newActor;
   }

   /**
    * getImage
    *
    * <p>If the Motion has a non zero speed (it's moving), then return
    * the AnimationMotion's image. If there is no speed, return the still
    * Animation.</p>
    *
    * @return Image
    */
   /**
    * <p>If the Motion has a non zero speed (it's moving), then return
    * the AnimationMotion's image. If there is no speed, return the still
    * Animation.</p>
    * @param xOffset
    * @param yOffset
    */
   @Override
   public void draw(int xOffset, int yOffset) {
      int x = (int)Math.round(getXLocation()) + xOffset;
      int y = (int)Math.round(getYLocation()) + yOffset;
//    if (motion.isMoving()) {
      motion[currentMotion].draw(x, y);
//    }
//    else {
//      return getStillAnimation().getImage();
//    }
   }

   /**
    * getMotion
    *
    * @return Motion
    */
   public Motion getMotion() {
      return motion[currentMotion];
   }

   /**
    * getStillAnimation
    *
    * @return Animation
    */
   public Animation getStillAnimation() {
      return stillAnimation;
   }

   /**
    * getMovement
    *
    * @return Movement
    */
   public Movement getMovement() {
      return movement;
   }

   /**
    * setMotion
    *
    * @param motion Motion
    */
   public void setMotion(Motion motion) {
      if (this.motion == null) {
         this.motion = new Motion[1];
      }
      this.motion[currentMotion] = motion;
   }

   /**
    * setStillAnimation
    *
    * @param stillAnimation Animation
    */
   public void setStillAnimation(Animation stillAnimation) {
      this.stillAnimation = stillAnimation;
   }

   /**
    * setMovement
    *
    * @param movement Movement
    */
   public void setMovement(Movement movement) {
      this.movement = movement;
   }

   /**
    * getDirection
    *
    * @return float
    */
   public float getDirection() {
      return movement.getGameVector().getDirection();
   }

   /**
    * setDirection
    *
    * <p>This is a cover method, to set the direction in the Animation
    * Movement.</p>
    *
    * @param direction float
    */
   public void setDirection(float direction) {
      motion[currentMotion].setDirection(direction);
   }

   /**
    * getSpeed
    *
    * @return double
    */
   public double getSpeed() {
      return movement.getGameVector().getSpeedXY();
   }

   /**
    * setCollisionMap
    *
    * @param collisionMap CollisionMap
    */
   public void setCollisionMap(CollisionMap collisionMap) {
      this.collisionMap = collisionMap;
   }

   protected abstract void spriteCollison(Sprite collideSprite);

}
